@@ -1,8 +1,10 @@
 package KiokuDB::Backend::MongoDB;
+BEGIN {
+  $KiokuDB::Backend::MongoDB::VERSION = '0.03';
+}
 use Moose;
 
 use namespace::clean -except => 'meta';
-our $VERSION = '0.02';
 
 with qw(
          KiokuDB::Backend
@@ -10,12 +12,9 @@ with qw(
          KiokuDB::Backend::Role::Clear
          KiokuDB::Backend::Role::Scan
          KiokuDB::Backend::Role::Query::Simple
+         KiokuDB::Backend::Role::Query
 );
 
-# TODO: 
-#    KiokuDB::Backend::Role::Scan
-#     (for some reason then all_entries doesn't always return all entries)
-
 use MongoDB::Connection; # In case we are expected to create the connection
 use Data::Stream::Bulk::Callback ();
 
@@ -55,16 +54,8 @@ sub clear {
 }
 
 sub all_entries {
-    my $self   = shift;
-    my $cursor = $self->collection->query({});
-    Data::Stream::Bulk::Callback->new(
-        callback => sub {
-            if (my $obj = $cursor->next) {
-                return [$self->deserialize($obj)];
-            }
-            return;
-        }
-    );
+    my $self = shift;
+    return $self->_proto_search({});
 }
 
 sub insert {
@@ -120,19 +111,30 @@ sub exists {
 
 sub simple_search {
     my ($self, $proto) = @_;
+    return $self->search($proto);
+}
+
+sub search {
+    my ($self, $proto, $args) = @_;
 
     for my $key (keys %$proto) {
         next if $key =~ m/^data\./;
+		next if $key eq 'class';
         my $value = delete $proto->{$key};
-        $proto->{"data.$key"} = ( defined $value ? "$value" : $value );
+        $proto->{"data.$key"} = $value;
     }
 
-    my $cursor = $self->collection->query($proto);
+    return $self->_proto_search($proto, $args);
+}
+
+sub _proto_search {
+    my ($self, $proto, $args) = @_;
+    my $cursor = $self->collection->query($proto, $args);
     return Data::Stream::Bulk::Callback->new(
         callback => sub {
             if (my $obj = $cursor->next) {
+				$obj->{_id} = $obj->{_id}->to_string if (ref $obj->{_id} eq 'MongoDB::OID');
                 return [$self->deserialize($obj)];
-                #return [$obj];
             }
             return;
         }
